#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import datetime
import subprocess
import os
import configparser

"""
Class: Versioning and messages of the wrapper
"""
class checker_body:

    def __init__(self, name):
        self.name = name
        self.version = "0.0.2"
        self.help_mssg()

    def __version__(self):
        string = "." * 80 + "\n"
        string += "\nData Quality Checker Wrapper Version: " + self.version
        print(string)

    def help_mssg(self):
        string = "\n."+ "." * 80 + "\n.\n"
        string += "."+ " " * 2+"Data Quality Checker Help message - How to run the DQC:\n.\n"
        string += "."+ " " * 4+"$ python checker.py <config_file.ini>\n.\n"
        string += "." + "." * 80 + "\n"

        self.help = string

"""
Check a parameter inside the config file (based on ConfigParser)
"""
def check_param_config(config, section, parameter):
    try:
        return config[section][parameter]         
    except:        
        raise Exception('Parameter {p} not defined. Check configuration file section [{s}]'.format(s=section, p=parameter))
        return None    


"""
Class: Main interface to the checks
---
- Reads config file as unique commandline operator
- Loops over the different checks/sections
"""
def main(args):
    dqc = checker_body("testing")
    dqc.__version__()

    # Help display
    try:
        if len(args) > 1:
            raise Exception('Too many arguments')
        if (args[0] in ["--help","-h","help","sos","commands"]):
            raise Exception('Help message')
    except:
        print(dqc.help)
        sys.exit()        

    # Read configuration and check mandatory fields in the general section   
    config = None
    try:
        if os.path.exists(args[0]):        
            config = configparser.ConfigParser()
            config.read(args[0])
        else:
            raise Exception('Configuration file does not exist')

        path_files = check_param_config(config, 'general', 'input')
        fpattern = check_param_config(config, 'general', 'fpattern')
        log_dir = check_param_config(config, 'general', 'log_dir')
        typef = check_param_config(config, 'general', 'type')
        
        if not os.path.exists(path_files):  
            raise Exception('Input directory does not exist. Check configuration file section [general]')
        if not log_dir:  
            raise Exception('Log directory not defined. Check configuration file section [general]')

        # Not needed anymore
        config.remove_section('general')
    except Exception as e:
        print('ERROR: {}'.format(e))
        sys.exit()
        
    # For each section/test
    for s in config.sections(): 
        
        string = "Data Quality Checker [check={}]\n".format(s)        
        string += "." + "." * 80 + "\n"
        print(string)        

        # Each section is a different test available
        try:
            ## TEST 00
            if s == 'file_format':
                import test00_file_format as test
                file_format_ck = test.file_format_check(log_dir, path_files, fpattern)
                print(file_format_ck.return_extensions_format())

            ## TEST 01
            elif s == 'standard_compliance':            
                import test01_standard_compliance as test            
                standard_ck = test.standard_compliance_test(log_dir, path_files, fpattern, typef)
                standard_ck.main()

            ## TEST 02
            elif s == 'spatial_completeness':            
                import test02_spatial_completeness as test
                spatial_completeness_ck = test.spatial_completeness_test(log_dir, path_files, fpattern, typef)
                spatial_completeness_ck.main()

            ## TEST 03
            elif s == 'temporal_completeness': 
                forms_dir = check_param_config(config, s, 'forms_dir')
                dataset = check_param_config(config, s, 'dataset')
                variable = check_param_config(config, s, 'variable')
                origin = check_param_config(config, s, 'origin')
                system = check_param_config(config, s, 'system')               
                import test03_temporal_completeness as test            
                temporal_completeness_ck = test.temporal_completeness_test(
                    log_dir, path_files, fpattern, forms_dir, dataset, variable, origin, system)
                temporal_completeness_ck.main()

            ## TEST 04
            elif s == 'spatial_consistency':              
                grid_int = check_param_config(config, s, 'grid_interval')
                grid_typ = check_param_config(config, s, 'grid_type')
                import test04_spatial_consistency as test
                spatial_consistency_ck = test.spatial_consistency_test(log_dir, path_files, fpattern, grid_typ, grid_int)
                spatial_consistency_ck.main()
            
            ## TEST 05
            elif s == 'temporal_consistency':
                time_s = check_param_config(config, s, 'time_step')
                time_g = check_param_config(config, s, 'time_granularity')
                import test05_temporal_consistency as test
                temporal_consistency_ck = test.temporal_consistency_test(log_dir, path_files, fpattern, time_s, time_g)  
                temporal_consistency_ck.main()          

            ## TEST 06
            elif s == 'valid_ranges':
                import test06_valid_ranges as test
                valid_ranges_ck = test.valid_ranges_test(log_dir, path_files, fpattern)
                valid_ranges_ck.main()            

            else:
                raise Exception('Test {} is not available in the current checker implementation. Please check your config syntax')

        except Exception as e:
            print('ERROR: Processing section {}'.format(s))
            print('ERROR: {}'.format(e))


if __name__ == "__main__":
    args = sys.argv[1:]
    sys.exit(main(args))
