#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import json
import subprocess
from datetime import datetime
from cdo import Cdo

# Local import
from utils.utils_files import *

"""
Class: Temporal completeness checks
---
- Uses CDO python bindings
- Loops over all timestamps within the dataset
- Checks they are within the time range [start/end] defined as input
"""
class temporal_completeness_test:

    def __init__(self, logdir, path, pattern, forms_dir, dataset, variable, origin, system):
        self.testid = 'test03-temporal-completeness'
        self.logdir = logdir 
        self.path = path        
        self.files = dir_or_file(path, pattern)
        self.forms_dir = forms_dir 
        self.dataset = dataset
        self.variable = variable        
        self.origin = origin
        self.system = system

    def get_forms_dates(self):
        constraints = {}
        
        # Load constraints file into dictionary
        try:            
            with open(os.path.join(self.forms_dir, self.dataset + '/constraints.json'), 'r') as cdf:
                constraints = json.load(cdf)                
        except:
            raise Exception('Problem loading constraints from cds-forms directory. Please check your configuration')

        # In particular dates for a given (dataset, variable)
        md = []
        try:
            dates = []
            for c in constraints:

                # Filter by variable
                if self.variable in c['variable']:

                    # Seasonal needs filter by origin/system
                    if 'seasonal' in self.dataset:
                        if not(self.origin in c['originating_centre']) or not(self.system in c['system']):
                            continue

                    # Parse all dates
                    for y in c['year']:
                        for m in c['month']:
                            for d in c['day']:
                                dates.append(datetime(int(y), int(m), int(d)))

            # Get unique ordered values
            md = sorted(list(set(dates)))
        except Exception as e:
            print(e)
            raise Exception('Problem loading constraints from cds-forms directory. Please check your dataset name')

        return md

    def check_timestamps(self):

        # Get mandatory dates from cds-forms
        mandatory_dates = self.get_forms_dates()
        dates_available = []

        # For all files
        cdo = Cdo()
        res = True
        
        # Progress init
        i = 0
        n = len(self.files)
        print()

        # Scan for available dates
        for f in self.files:

            # Progress update
            i+=1
            print_progress(i, n)                          
            logging.info('Checking {}'.format(f))

            try:
                info = cdo.showtimestamp(input=f)
            except:
                logging.error('Bad file format (cdo cannot read file)')
                print('ERROR: Bad file format ({})'.format(os.path.basename(f)))
                res = False
                break

            # Gather available timestamps
            for inf in info:                
                for t in inf.split():                    
                    dates_available.append(
                        datetime.strptime(t, '%Y-%m-%dT%H:%M:%S').replace(hour=0, minute=0, second=0)
                    )

        # Check consistency
        uq_dates_available = sorted(list(set(dates_available)))
        for md in mandatory_dates:            
            if not(md in uq_dates_available):
                res = False
                logging.error('Date {} was not found in the provided directory. Stopping check'.format(md))
                break            

        # Final message
        if res:
            logging.info('FILES ARE TEMPORALLY COMPLETE')
            print('INFO: FILES ARE TEMPORALLY COMPLETE')
        else:
            logging.error('TEMPORAL INCOMPLETENESS FOUND')
            print('INFO: TEMPORAL INCOMPLETENESS FOUND')

    def main(self):
        # Initiate the appropiate logs
    	initiate_logging(self.logdir, self.path, self.testid)

        # Do the actual work
    	self.check_timestamps()
                
if __name__ == "__main__":
    sys.exit()
