#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Ivan Jose Cernicharo Ortiz (ivan.cernicharo@bsc.es)

import sys
import logging
import os
import subprocess
from datetime import date, datetime, timedelta
from calendar import monthrange
from cdo import Cdo
import numpy as np

# Local imports
from utils.utils_files import *

"""
Class: Spatial consistency checks
---
- Check frequency/leadtimes of a given variable
"""
class temporal_consistency_test:

    def __init__(self, logdir, path, pattern, time_step, time_granularity):
        self.testid = 'test05-temporal-consistency'
        self.path = path
        self.logdir = logdir
        self.time_s = int(time_step)
        self.time_g = time_granularity.lower()                
        self.files = dir_or_file(path, pattern)
            
    """
    Difference between two dates in months (takes into account calendar)
    """
    def monthdelta(self, d1, d2):
        delta = 0
        while True:
            mdays = monthrange(d1.year, d1.month)[1]
            d1 += timedelta(days=mdays)
            if d1 <= d2:
                delta += 1
            else:
                break
        return delta

    """
    Compute time difference given t1,t2 datetime objects and conditions (time_unit and granularity)
    """
    def get_time_difference(self, t1, t2):

      # Substract datetimes
      tdiff = (t2 - t1).total_seconds()          
      if tdiff == 0:
        return True # grib messages have multiple timestamps (same date)

      # Depending on granularity
      if self.time_g == 'M' or 'month' in self.time_g.lower():        
        return self.monthdelta(t1, t2) == self.time_s
      elif self.time_g == 'D' or 'day' in self.time_g.lower():        
        return (tdiff / 86400) == self.time_s
      elif self.time_g == 'H' or 'hour' in self.time_g.lower():
        return (tdiff / 3600) == self.time_s
      elif self.time_g == 'S' or 'sec' in self.time_g.lower():
        return (tdiff) == self.time_s            
      else:
        raise Exception('Please verify the time_granularity parameter (Month, Day, Hour, Sec)')

    def check_timestamps(self):

        # For all files
        cdo = Cdo()
        res = True
        
        # Progress init
        i = 0
        n = len(self.files)
        print()

        for f in self.files:
            # Progress update
            i+=1
            print_progress(i, n)              
            logging.info('Checking {}'.format(f))       

            try:
                info = cdo.showtimestamp(input=f)
            except:
                logging.error('Bad file format (cdo cannot read file)')
                print('ERROR: Bad file format ({})'.format(os.path.basename(f)))
                res = False
                break

            # Collect all timestamps
            dates = []
            for inf in info:                
                for t in inf.split():
                    dates.append(datetime.strptime(t, '%Y-%m-%dT%H:%M:%S'))

            # Sort and seek
            d0 = False            
            for d1 in sorted(dates):                            
              if d0:                
                res = res & self.get_time_difference(d0, d1)
                if not(res):
                  logging.error('Temporal inconsistency detected [t0={} - t1={}]'.format(d0, d1))                     
                  break           
              d0 = d1

            # Break outer loop
            if not(res): break

        # Final message
        if res:
            logging.info('FILES ARE TEMPORALLY CONSISTENT')
            print('INFO: FILES ARE TEMPORALLY CONSISTENT')
        else:
            logging.error('TEMPORAL INCONSISTENCY FOUND')
            print('INFO: TEMPORAL INCONSISTENCY FOUND')

    def main(self):
        # Initiate the appropiate logs
      initiate_logging(self.logdir, self.path, self.testid)

      self.check_timestamps()
        

if __name__ == "__main__":
    sys.exit()
